@@ -17,8 +17,6 @@ cpuidle_states_path = "/sys/devices/system/cpu/cpu0/cpuidle"
 
 class CPULatencyPlugin(hotplug.Plugin):
 	"""
-	`cpu`::
-	
 	Sets the CPU governor to the value specified by the [option]`governor`
 	option and dynamically changes the Power Management Quality of
 	Service (PM QoS) CPU Direct Memory Access (DMA) latency according
@@ -30,11 +28,6 @@ class CPULatencyPlugin(hotplug.Plugin):
 	character is meant to represent a logical 'or' operator. Note that the
 	same syntax is used for the [option]`energy_perf_bias` option. *TuneD*
 	will set the first governor that is available on the system.
-	+    
-	For example, with the following profile, *TuneD* will set the 'ondemand'
-	governor, if it is available. If it is not available, but the 'powersave'
-	governor is available, 'powersave' will be set. If neither of them are
-	available, the governor will not be changed.
 	+
 	.Specifying a CPU governor
 	====
@@ -42,6 +35,11 @@ class CPULatencyPlugin(hotplug.Plugin):
 	[cpu]
 	governor=ondemand|powersave
 	----
+
+	*TuneD* will set the 'ondemand'
+	governor, if it is available. If it is not available, but the 'powersave'
+	governor is available, 'powersave' will be set. If neither of them are
+	available, the governor will not be changed.
 	====
 	
 	`sampling_down_factor`:::
@@ -104,10 +102,10 @@ class CPULatencyPlugin(hotplug.Plugin):
 	`latency_low, latency_high, load_threshold`:::
 	+
 	If the CPU load is lower than the value specified by
-	the[option]`load_threshold` option, the latency is set to the value
+	the [option]`load_threshold` option, the latency is set to the value
 	specified either by the [option]`latency_high` option or by the
 	[option]`latency_low` option.
-	+
+
 	`force_latency`:::
 	You can also force the latency to a specific value and prevent it from
 	dynamically changing further. To do so, set the [option]`force_latency`
@@ -115,14 +113,16 @@ class CPULatencyPlugin(hotplug.Plugin):
 	+
 	The maximum latency value can be specified in several ways:
 	+
-	 * by a numerical value in microseconds (for example, `force_latency=10`)
-	 * as the kernel CPU idle level ID of the maximum C-state allowed
-	   (for example, force_latency = cstate.id:1)
-	 * as a case sensitive name of the maximum C-state allowed
-	   (for example, force_latency = cstate.name:C1)
-	 * by using 'None' as a fallback value to prevent errors when alternative
-	   C-state IDs/names do not exist. When 'None' is used in the alternatives
-	   pipeline, all the alternatives that follow 'None' are ignored.
+	--
+	* by a numerical value in microseconds (for example, `force_latency=10`)
+	* as the kernel CPU idle level ID of the maximum C-state allowed
+	  (for example, force_latency = cstate.id:1)
+	* as a case sensitive name of the maximum C-state allowed
+	  (for example, force_latency = cstate.name:C1)
+	* by using 'None' as a fallback value to prevent errors when alternative
+	  C-state IDs/names do not exist. When 'None' is used in the alternatives
+	  pipeline, all the alternatives that follow 'None' are ignored.
+	--
 	+
 	It is also possible to specify multiple fallback values separated by '|' as
 	the C-state names and/or IDs may not be available on some systems.
@@ -146,8 +146,8 @@ class CPULatencyPlugin(hotplug.Plugin):
 	[cpu]
 	force_latency=cstate.name:XYZ|None
 	----
-	In this case, if C-state with the name `XYZ` does not exist
-	[option]`force_latency`, no latency value will be written into the
+	In this case, if C-state with the name `XYZ` does not exist,
+	no latency value will be written into the
 	kernel's PM QoS file, and no errors will be reported due to the
 	presence of 'None'.
 	====
@@ -165,31 +165,40 @@ class CPULatencyPlugin(hotplug.Plugin):
 	Limit the minimum P-State that will be requested by the driver. It states
 	it as a percentage of the max (non-turbo) performance level.
 	====
-	+
+
 	`pm_qos_resume_latency_us`:::
 	This option allow to set specific latency for all cpus or specific ones.
+	+
+	.Configuring resume latency
 	====
 	----
 	[cpu]
 	pm_qos_resume_latency_us=n/a
 	----
 	Special value that disables C-states completely.
-	====
 	----
 	[cpu]
 	pm_qos_resume_latency_us=0
 	----
 	Allows all C-states.
-	====
 	----
 	[cpu]
 	pm_qos_resume_latency_us=100
 	----
-	Allows any C-state with a resume latency less than value.
+	Allows any C-state with a resume latency less than 100.
+	====
+
+	`boost`:::
+	The [option]`boost` option allows the CPU to boost above nominal
+	frequencies for shorts periods of time.
+	+
+	.Allowing CPU boost
+	====
+	----
 	[cpu]
 	boost=1
 	----
-	CPU is allowed to boost above nominal frequencies for short periods of time.
+	====
 	"""
 
 	def __init__(self, *args, **kwargs):
@@ -397,8 +406,8 @@ class CPULatencyPlugin(hotplug.Plugin):
 				instance.options["max_perf_pct"])
 			self._max_perf_pct_save = self._getset_intel_pstate_attr(
 				"max_perf_pct", new_value)
-			new_value = self._variables.expand(
-				instance.options["no_turbo"])
+			new_value = self._cmd.get_bool(self._variables.expand(
+				instance.options["no_turbo"]))
 			self._no_turbo_save = self._getset_intel_pstate_attr(
 				"no_turbo", new_value)
 
@@ -517,7 +526,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return self._cmd.read_file("/sys/devices/system/cpu/%s/cpufreq/scaling_available_governors" % device).strip().split()
 
 	@command_set("governor", per_device=True)
-	def _set_governor(self, governors, device, sim, remove):
+	def _set_governor(self, governors, device, instance, sim, remove):
 		if not self._check_cpu_can_change_governor(device):
 			return None
 		governors = str(governors)
@@ -546,7 +555,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return governor
 
 	@command_get("governor")
-	def _get_governor(self, device, ignore_missing=False):
+	def _get_governor(self, device, instance, ignore_missing=False):
 		governor = None
 		if not self._check_cpu_can_change_governor(device):
 			return None
@@ -563,7 +572,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return "/sys/devices/system/cpu/cpufreq/%s/sampling_down_factor" % governor
 
 	@command_set("sampling_down_factor", per_device = True, priority = 10)
-	def _set_sampling_down_factor(self, sampling_down_factor, device, sim, remove):
+	def _set_sampling_down_factor(self, sampling_down_factor, device, instance, sim, remove):
 		val = None
 
 		# hack to clear governors map when the profile starts unloading
@@ -590,7 +599,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return val
 
 	@command_get("sampling_down_factor")
-	def _get_sampling_down_factor(self, device, ignore_missing=False):
+	def _get_sampling_down_factor(self, device, instance, ignore_missing=False):
 		governor = self._get_governor(device, ignore_missing=ignore_missing)
 		if governor is None:
 			return None
@@ -618,7 +627,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return "/sys/devices/system/cpu/cpu%s/power/energy_perf_bias" % cpu_id
 
 	@command_set("energy_perf_bias", per_device=True)
-	def _set_energy_perf_bias(self, energy_perf_bias, device, sim, remove):
+	def _set_energy_perf_bias(self, energy_perf_bias, device, instance, sim, remove):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
@@ -697,7 +706,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 				}.get(self._try_parse_num(s), s)
 
 	@command_get("energy_perf_bias")
-	def _get_energy_perf_bias(self, device, ignore_missing=False):
+	def _get_energy_perf_bias(self, device, instance, ignore_missing=False):
 		energy_perf_bias = None
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
@@ -732,7 +741,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return self._has_pm_qos_resume_latency_us
 
 	@command_set("pm_qos_resume_latency_us", per_device=True)
-	def _set_pm_qos_resume_latency_us(self, pm_qos_resume_latency_us, device, sim, remove):
+	def _set_pm_qos_resume_latency_us(self, pm_qos_resume_latency_us, device, instance, sim, remove):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
@@ -748,7 +757,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return latency
 
 	@command_get("pm_qos_resume_latency_us")
-	def _get_pm_qos_resume_latency_us(self, device, ignore_missing=False):
+	def _get_pm_qos_resume_latency_us(self, device, instance, ignore_missing=False):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
@@ -757,7 +766,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return self._cmd.read_file(self._pm_qos_resume_latency_us_path(device), no_error=ignore_missing).strip()
 
 	@command_set("boost", per_device=True)
-	def _set_boost(self, boost, device, sim, remove):
+	def _set_boost(self, boost, device, instance, sim, remove):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
@@ -776,7 +785,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return None
 
 	@command_get("boost")
-	def _get_boost(self, device, ignore_missing=False):
+	def _get_boost(self, device, instance, ignore_missing=False):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
@@ -788,7 +797,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return None
 
 	@command_set("energy_performance_preference", per_device=True)
-	def _set_energy_performance_preference(self, energy_performance_preference, device, sim, remove):
+	def _set_energy_performance_preference(self, energy_performance_preference, device, instance, sim, remove):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
@@ -798,7 +807,13 @@ class CPULatencyPlugin(hotplug.Plugin):
 			if not sim:
 				avail_vals = set(self._cmd.read_file(self._pstate_preference_path(cpu_id, True)).split())
 				for val in vals:
-					if val in avail_vals:
+					try:
+						val = int(val)
+						valid = 0 <= val < 256
+						val = str(val)
+					except ValueError:
+						valid = val in avail_vals
+					if valid:
 						self._cmd.write_to_file(self._pstate_preference_path(cpu_id), val, \
 							no_error = [errno.ENOENT] if remove else False, ignore_same=True)
 						log.info("Setting energy_performance_preference value '%s' for cpu '%s'" % (val, device))
@@ -814,7 +829,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return None
 
 	@command_get("energy_performance_preference")
-	def _get_energy_performance_preference(self, device, ignore_missing=False):
+	def _get_energy_performance_preference(self, device, instance, ignore_missing=False):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
